#!/usr/bin/env python

"""
The main script that runs the code, gets the Spotify API access, downloads the album art and data
"""

import os
import os.path
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


from .get_spotify_access import get_spotify_credentials
from .get_album_details import get_album_details
from .get_album_art import get_album_art
from .get_album_data import get_album_audio_analysis, get_album_audio_features

from tqdm import tqdm


def run_all(artist_name, client_id, client_secret):
    """
    Runs all the methods to download all the album art and info for the specified artist

    :param artist_name: Name of artist
    :type artist_name: str
    :param client_id: Client ID for Spotify API Access
    :type client_id: str
    :param client_secret: Client Secret for Spotify API Access
    :type client_secret: str
    :returns: None
    """

    parent_dir = "./data"
    artist_dir = f"{artist_name}"
    base_path = os.path.join(parent_dir, artist_dir)

    # Define save-path for album art

    album_cover_dir = "Album_Art/"
    album_cover_path = os.path.join(base_path, album_cover_dir)
    try:
        os.makedirs(album_cover_path)
    except FileExistsError:
        pass

    # Define save-path for album analysis audio frames

    album_info_dir = "Album_Info/"
    album_info_path = os.path.join(base_path, album_info_dir)
    try:
        os.makedirs(album_info_path)
    except FileExistsError:
        pass

    sp = get_spotify_credentials(client_id=client_id, client_secret=client_secret)

    album_names, album_name_uri_dict, album_img_url_dict = get_album_details(sp=sp, artist_name=artist_name)

    for album_name in tqdm(album_names):

        get_album_art(album_name=album_name, album_img_url_dict=album_img_url_dict,
                      album_cover_path=album_cover_path)

    for album_name in tqdm(album_names):

        get_album_audio_features(sp=sp, album_name=album_name, album_name_dict=album_name_uri_dict,
                                 album_info_path=album_info_path)

        get_album_audio_analysis(sp=sp, album_name=album_name, album_name_dict=album_name_uri_dict,
                                 album_info_path=album_info_path)



"""
This script downloads all the album data (audio features and audio analysis) for an artist from Spotify
"""

import os.path
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from tqdm import tqdm
import pandas as pd


def get_album_audio_features(sp, album_name, album_name_dict, album_info_path):
    """Get audio features data for all albums for the given artists and pickle the data-frames

    :param sp: Spotify object
    :type sp: object
    :param album_name: List of album names
    :type album_name: list
    :param album_name_dict: Dictionary mapping album names to album uri values
    :type album_name_dict: dict
    :param album_info_path: Path to save pickle files to
    :type album_info_path: str
    :returns: df_audio_features: Data-frame of audio features for album
    :rtype: data-frame
    """

    print(f"Creating and pickling audio features data-frame for {album_name}")

    # Get album tracks

    album_uri = album_name_dict[album_name]
    tracks = sp.album_tracks(album_uri)

    album_data = pd.json_normalize(tracks["items"])
    album_data = album_data[["duration_ms", "name", "track_number", "uri"]]

    # Get audio features for each track and build a data frame

    df_audio_features = pd.DataFrame()

    for name, uri in tqdm(
        list(zip(album_data["name"].tolist(), album_data["uri"].tolist()))
    ):
        track_audio_features = pd.json_normalize(sp.audio_features(uri))
        track_audio_features["name"] = name
        df_audio_features = pd.concat(
            [df_audio_features, track_audio_features], axis=0
        )

    df_audio_features.set_index("name", inplace=True)
    df_audio_features.reset_index(inplace=True)

    df_audio_features.to_pickle(
        f"{album_info_path}/{album_name}_audio_features.pkl"
    )

    return df_audio_features


def get_album_audio_analysis(sp, album_name, album_name_dict, album_info_path):
    """Get audio analysis data for all albums for the given artists and pickle the data-frames

    :param sp: Spotify object
    :type sp: object
    :param album_name: List of album names
    :type album_name: list
    :param album_name_dict: Dictionary mapping album names to album uri values
    :type album_name_dict: dict
    :param album_info_path: Path to save pickle files to
    :type album_info_path: str
    :returns: df_audio_analysis: Data-frame for audio analysis of album
    :rtype: data-frame

    """

    # Get album tracks

    album_uri = album_name_dict[album_name]
    tracks = sp.album_tracks(album_uri)

    album_data = pd.json_normalize(tracks["items"])
    album_data = album_data[["duration_ms", "name", "track_number", "uri"]]

    # Get audio analysis for each track and build a data frame

    print(f"Creating and pickling audio analysis data-frame for {album_name}")

    df_audio_analysis = pd.DataFrame()

    for name, uri in tqdm(
        list(zip(album_data["name"].tolist(), album_data["uri"].tolist()))
    ):
        track_audio_analysis = pd.json_normalize(sp.audio_analysis(uri))
        track_audio_analysis["name"] = name
        df_audio_analysis = pd.concat(
            [df_audio_analysis, track_audio_analysis], axis=0
        )

    df_audio_analysis.set_index("name", inplace=True)
    df_audio_analysis.reset_index(inplace=True)

    df_audio_analysis.to_pickle(
        f"{album_info_path}/{album_name}_audio_analysis.pkl"
    )

    return df_audio_analysis



"""
This script downloads all the album art (high resolution) for an artist from Spotify
"""

import os.path
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

import urllib.request
from PIL import Image


def get_album_art(album_name, album_img_url_dict, album_cover_path):
    """Downloads and saves highest resolution album artwork for the specified album

    :param album_name: Album name
    :type album_name: str
    :param album_img_url_dict: Dictionary mapping album names to album cover urls
    :type album_img_url_dict: dict
    :param album_cover_path: Path to save album images to
    :type album_cover_path: str
    :returns: Album cover
    :rtype: image file

    """

    print(f"Downloading album cover for {album_name}")

    # Get album cover
    url = album_img_url_dict[album_name]
    image = Image.open(urllib.request.urlopen(url))

    image.save(f"{album_cover_path}/{album_name}_art.png")

    return image




#!/usr/bin/env python

"""
Access script
"""

import os.path
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from access_spotify import run_all
import argparse

parser = argparse.ArgumentParser(description='Get artist name for API access')
parser.add_argument(
    '--artist_name', '-a',
    default='The Beatles',
    help='Artist to download all album art and info for'
)
parser.add_argument(
    '--client_id', '-id',
    required=True,
    help='Client ID for API'
)
parser.add_argument(
    '--client_secret', '-s',
    required=True,
    help='Client Secret for API'
)

my_namespace = parser.parse_args()
artist_name = my_namespace.artist_name
CLIENT_ID = my_namespace.client_id
CLIENT_SECRET = my_namespace.client_secret


if __name__ == "__main__":
    run_all.run_all(artist_name=artist_name, client_id=CLIENT_ID, client_secret=CLIENT_SECRET)





"""
This script gets the details for all of the specified artist's albums

"""

import os.path
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


def get_album_details(sp, artist_name):
    """Gets details for all of the albums for the specified artist

    :param sp: Spotify object
    :type sp: object
    :param artist_name: Artist to query for
    :type artist_name: str
    :returns: album_names: List of album names
        album_name_dict: Dictionary mapping album names to album uri values
        album_img_url_dict: Dictionary mapping album names to album cover urls
    :rtype: (list, dict, dict)

    """

    result = sp.search(artist_name)  # search query

    # Extract Artist's uri
    artist_uri = result["tracks"]["items"][0]["artists"][0]["uri"]

    # Pull all of the artist's albums
    sp_albums = sp.artist_albums(artist_uri, album_type="album")
    # CHECK THAT ALL THE ALBUMS ARE BEING DOWNLOADED -- MIGHT BE A BUG
    # Store artist's albums' names' and uris in separate lists
    album_names = []
    album_uris = []
    album_img_urls = []
    for i in range(len(sp_albums["items"])):
        album_names.append(sp_albums["items"][i]["name"])
        album_uris.append(sp_albums["items"][i]["uri"])
        album_img_urls.append(sp_albums["items"][i]["images"][0]["url"])

    album_name_dict = dict(zip(album_names, album_uris))
    album_img_url_dict = dict(zip(album_names, album_img_urls))

    return album_names, album_name_dict, album_img_url_dict




#!/usr/bin/env python

import setuptools
from setuptools import setup

with open("requirements.txt", "r") as f:
    requirements = [line.rstrip("\n") for line in f if line != "\n"]

setup(name='access-spotify',
      version="1.0",
      author="pancham_banerjee",
      author_email="panchajanya.banerjee@gmail.com",
      packages=setuptools.find_packages(),
      scripts=["./bin/access_script.py"],
      install_requires=requirements,
      license="MIT",
      description="A package to get all album and track info for an artist by querying the Spotify API"
      )




#!/usr/bin/env python

"""
This script uses the user's Client ID and Client Secret to get access to the Spotify ID
"""

# API Credentials

import os.path
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

import spotipy
from spotipy.oauth2 import SpotifyClientCredentials


def get_spotify_credentials(client_id, client_secret):
    """Takes in API credentials to start querying the Spotify API

    :param client_id: Spotify Client ID
    :type client_id: str
    :param client_secret: Spotify Client Secret
    :type client_secret: str
    :returns: sp: Spotify object
    :rtype: object

    """

    client_id = client_id
    client_secret = client_secret

    client_credentials_manager = SpotifyClientCredentials(
        client_id=client_id, client_secret=client_secret
    )
    sp = spotipy.Spotify(client_credentials_manager=client_credentials_manager)

    return sp
